@@ -289,7 +289,7 @@ def changeMiniBosses(rom, mapping):
         ld  hl, $C17B
         or  [hl]
         ret nz
-        ldh a, [$F6] ; room
+        ldh a, [$FFF6] ; room
         cp  $45 ; check for D3 dodogo room
         ret z
         cp  $7F ; check for armos temple room
@@ -403,11 +403,11 @@ def doubleTrouble(rom):
         #     re.entities += [(3, 4, 0x63), (2, 4, 0x63)]
         #     re.store(rom)
         #     # Remove that links movement is blocked
-        #     rom.patch(0x05, 0x2258, ASM("ldh [$A1], a"), "0000")
-        #     rom.patch(0x05, 0x1AE3, ASM("ldh [$A1], a"), "0000")
-        #     rom.patch(0x05, 0x1C5D, ASM("ldh [$A1], a"), "0000")
-        #     rom.patch(0x05, 0x1C8D, ASM("ldh [$A1], a"), "0000")
-        #     rom.patch(0x05, 0x1CAF, ASM("ldh [$A1], a"), "0000")
+        #     rom.patch(0x05, 0x2258, ASM("ldh [$FFA1], a"), "0000")
+        #     rom.patch(0x05, 0x1AE3, ASM("ldh [$FFA1], a"), "0000")
+        #     rom.patch(0x05, 0x1C5D, ASM("ldh [$FFA1], a"), "0000")
+        #     rom.patch(0x05, 0x1C8D, ASM("ldh [$FFA1], a"), "0000")
+        #     rom.patch(0x05, 0x1CAF, ASM("ldh [$FFA1], a"), "0000")
         if re.hasEntity(0x62):  # hot head (TODO: Drops thwo hearts)
             re.removeEntities(0x62)
             re.entities += [(2, 2, 0x62), (4, 4, 0x62)]
